# <trigger>
# 	<usage>perl checkSiblingsandTransitionParent_noSecurity.pl parentReportName childReportName stateName attributeName validValues</usage>
# 	<version>4.4</version>
# 	<description>该触发器用于当所有子 CR 都处于指定状态时，自动转变父 CR。对于指定的子 CR，运行报告以获取父 CR。现在运行报告以获取指定子 CR 的所有同代。如果所有同代均处于有效状态，那么就可以转变父 CR。
#		该触发器在 ALM 生命周期的多个不同阶段使用。检查是否可以转变 BCR 就是一个示例。运行的父报告是 Data_Report_RCP_BCR，运行的子报告是 Data_Report_BCR_RCP。运行子报告以获取所有同代的列表。要转变为的状态的名称是“BCR 正在进行中”，要检查的属性是 crstatus。有效值为“RCP 已应用”和“RCP 被拒绝”。如果所有 RCP 均处于正确的状态，那么就可以将 BCR 转变为具有所提供名称的状态。
#		要检查的属性必须出现在作为参数传递的报告的报告定义中。该触发器是转变后触发器。这种方法不具有安全性，这意味着触发器会生成转变名称，而不使用提供给它的转变名称。
#		在 ALM 生命周期中，这种转变是隐藏式的，因为它只会由触发器自动执行。
#    </description>
# 	<argument1>
# 		<name>parentReportName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>要在触发期间运行的父报告的名称。</description>
#        </argument1>
# 	<argument2>
# 		<name>childReportName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>要在触发期间运行的子报告的名称。</description>
#        </argument2>
# 	<argument3>
# 		<name>stateName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>要转变为的状态的名称。</description>
#        </argument3>
# 	<argument4>
# 		<name>attributeName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>要检查的属性的名称。</description>
#        </argument4>
# 	<argument5>
# 		<name>validValues</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>要检查的属性的有效值，以分号分隔。属性必须是指定的有效值之一，否则将不会转变 CR。
#                </description>
#        </argument4>
#  </trigger>

use strict;
use utf8;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;

my $parentreport = shift;
my $childreport = shift;
my $stateName = shift;
my $attrToCheck = shift;
my $attrValueList = shift;

my @validAttrValues = split /;/, $attrValueList;

#Create a new TriggerParser object, passing in the location of the trigger xml document.
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

#Create a new csapi object.
my $csapi = new ChangeSynergy::csapi();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(), $trigger->get_token(), $trigger->get_database());

#Create a query string and then call the api to get a data report.
my $queryString = "problem_number='" . $trigger->get_object_id() . "'";

my $data = $csapi->QueryData($user, $parentreport, $queryString, undef, undef, undef);

#Get the child data, this is the associated data, the sub-report data.
my $childData = $data->getDataObject(0)->getChildData();

#For each csapi_object_vector_position under the csapi_cobject_vector_assoc tag.
for(my $i = 0; $i < $childData->getDataSize(); $i++)
{
	#Get the object vector
	my $objVec = $childData->getDataObject($i);

	# Run a query to get the child data of the parent.  This is to find all the siblings of the initial object
	my $problem_number = $objVec->getDataObjectByName("problem_number")->getValue();
	my $subqueryString = "problem_number='" . $problem_number . "'";

	my $subdata = $csapi->QueryData($user, $childreport, $subqueryString, undef, undef, undef);
	
	# Now that we have the parent object, we find all of its children. These are the siblings.
	my $sibling = $subdata->getDataObject(0)->getChildData();
	for (my $j=0; $j < $sibling->getDataSize(); $j++)
	{
		my $siblingData = $sibling->getDataObject($j);
		my $match = 0;
		# Check to make sure the attribute to check has one of the valid values, if it doesn't, the script exits
		# without performing the transition.
		foreach my $value (@validAttrValues)
		{
			if ($siblingData->getDataObjectByName($attrToCheck)->getValue() eq "\L$value")
			{
				$match = 1;
			}
		}
		if ($match == 0)
		{
			exit(0);
		}
	}

	#The name of the transition is the current state of the object, the string "2", and the state to transition to
	my $transitionName = "HIDDEN_" . $objVec->getDataObjectByName("crstatus")->getValue() . "2" . $stateName;

	#Get the transition show form.
	my $tmp = $csapi->TransitionCRData($user, $problem_number , $transitionName);	

	print "Automatically transitioned CR $problem_number to $stateName\n";

	#Set the crstatus to the state we want the CR to end up in.
	$tmp->getDataObjectByName("crstatus")->setValue($stateName);

	#Do the transition
	$csapi->TransitionCR($user, $tmp);	
}
